package com.contextspace.objectdb_example.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.jdo.annotations.Index;
import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.xml.bind.annotation.XmlTransient;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


@Entity
@Table
public class AttributeName extends IDHolder {
	private static final long serialVersionUID = 6789661029535292158L;
	
	static Logger log = LoggerFactory.getLogger(AttributeName.class.getName());
	
	/**
	 * The name that can be used as an attribute name 
	 */
	private String name = null;
	

	/**
	 * A map holding the display values in all languages for this attribute name
	 */
	private Map<String, String> displayValues = new HashMap<String, String>();
	
	
	/**
	 * Default constructor for DB loading.
	 */
	public AttributeName() {
		super();
	}
	
	
	/**
	 * Creates the name using the name. Doesn't associate the name with any organisation or category.
	 * @param name
	 */
	public AttributeName(String name) {
		super();
		this.name = name;
		displayValues.put("en", name);
	}
	
	/**
	 * Creates the name using the name and a default English display name <displayValue>. Doesn't associate the name with any organisation or category.
	 * @param name
	 * @param displayValue
	 */
	public AttributeName(String name, String displayValue) {
		super();
		this.name = name;
		displayValues.put("en", displayValue);
	}
	

	/**
	 * Gets the display name for this attribute in the language specified in <lang>. If no display value is stored for this language, the English display value will be returned.
	 * @param lang the language of the required display value
	 * @return
	 */
	public String getDisplayValue(String lang) {
		String displayValue = displayValues.get(lang);
		
		if(displayValue == null) {
			displayValue = displayValues.get("en");
		}
		
		return displayValue;
	}
	
	/**
	 * @return the English display value
	 */
	@Transient
	@XmlTransient
	public String getDisplayValue() {
		return displayValues.get("en");
	}
	
	/**
	 * Sets the display value for the specified language <lang>. If <displayVlue> is empty, the old display value for this language will be deleted.
	 * @param lang
	 * @param displayValue
	 */
	public void setDisplayValue(String lang, String displayValue) {
		if(lang == null || lang.equals("")) {
			log.error("A language must be specified when adding a display value.");
			return;
		}
		
		// empty displayValue means the language key is going to be removed
		if(displayValue == null || displayValue.equals("")) {
			displayValues.remove(lang);
			return;
		}
		
		displayValues.put(lang, displayValue);
	}
	
	/**
	 * Sets the default display value. the default locale is 'en'
	 * @param displayValue
	 */
	public void setDisplayValue(String displayValue) {
		if(displayValue == null || displayValue.equals("")) {
			log.error("A display value must be specified when setting a default display value.");
			return;
		}
		displayValues.put("en", displayValue);
	}
	
	/**
	 * @return the name
	 */
	@Index(unique="true")
	public String getName() {
		return name;
	}


	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}


	/**
	 * @return the displayValues
	 */
	@OneToMany(cascade=CascadeType.ALL, fetch=FetchType.EAGER, orphanRemoval=true)
	public Map<String, String> getDisplayValues() {
		return displayValues;
	}

	/**
	 * @param displayValues the displayValues to set
	 */
	public void setDisplayValues(Map<String, String> displayValues) {
		this.displayValues = displayValues;
	}

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuffer buf = new StringBuffer();
        
        StringBuffer langBuf = new StringBuffer();
        for(Iterator<String> i = displayValues.keySet().iterator(); i.hasNext();) {
            String lang = i.next();
            String value = displayValues.get(lang);
            if(langBuf.length() > 0) {
                langBuf.append(",");
            }
            langBuf.append(lang).append("=").append(value);
        }
        
        buf.append("<attributename name=\"").append(name).append("\"").append(" values=\"").append(langBuf.toString()).append("\"");
        if(getId() > 0) {
            buf.append(" id=\"").append(getId()).append("\"");
        }
        buf.append("/>");
        return buf.toString();
    }


    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result
                + ((displayValues == null) ? 0 : displayValues.hashCode());
        result = prime * result + ((name == null) ? 0 : name.hashCode());
        return result;
    }


    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
/*    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        AttributeName other = (AttributeName) obj;
        if (displayValues == null) {
            if (other.displayValues != null)
                return false;
        } else if (!displayValues.equals(other.displayValues))
            return false;
        if (name == null) {
            if (other.name != null)
                return false;
        } else if (!name.equals(other.name))
            return false;
        return true;
    }
*/

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        System.out.println("AttributeName EQUALS method called for " + this + " against: " + obj);
        if(!(obj instanceof AttributeName)) {
            return false;
        }
        AttributeName attrName = (AttributeName)obj;
        if(this.getId() != 0 && attrName.getId() != 0) {
            if(this.getId() == attrName.getId()) {
                return true;
            }
            else {
                return false;
            }
        }
        else if(this.getName().equals(attrName.getName())){
            return true;
        }
        return false;
    }


    
    
    
    
}
